@@ -6,6 +6,7 @@ module Hint.Annotations (
 import Data.Data
 import Annotations
 import GHC.Serialized
+import MonadUtils (concatMapM)
 
 import Hint.Base
 import HscTypes (hsc_mod_graph, ms_mod)
@@ -14,17 +15,15 @@ import qualified Hint.GHC as GHC
 -- Get the annotations associated with a particular module.
 getModuleAnnotations :: (Data a, MonadInterpreter m) => a -> String -> m [a]
 getModuleAnnotations _ x = do
-    mods <- fmap (GHC.mgModSummaries . hsc_mod_graph) $ runGhc GHC.getSession
+    mods <- GHC.mgModSummaries . hsc_mod_graph <$> runGhc GHC.getSession
     let x' = filter ((==) x . GHC.moduleNameString . GHC.moduleName . ms_mod) mods
-    v <- mapM (anns . ModuleTarget . ms_mod) x'
-    return $ concat v
+    concatMapM (anns . ModuleTarget . ms_mod) x'
 
 -- Get the annotations associated with a particular function.
 getValAnnotations :: (Data a, MonadInterpreter m) => a -> String -> m [a]
-getValAnnotations _ x = do
-    x' <- runGhc1 GHC.parseName x
-    v <- mapM (anns . NamedTarget) x'
-    return $ concat v
+getValAnnotations _ s = do
+    names <- runGhc1 GHC.parseName s
+    concatMapM (anns . NamedTarget) names
 
 anns :: (MonadInterpreter m, Data a) => AnnTarget GHC.Name -> m [a]
 anns = runGhc1 (GHC.findGlobalAnns deserializeWithData)
