@@ -1,29 +1,23 @@
 #if defined(mingw32_HOST_OS) || defined(__MINGW32__)
 {-# LANGUAGE ForeignFunctionInterface #-}
 #endif
-module Hint.CompatPlatform
-  (
+module Hint.CompatPlatform (
     getPID
-  )
-
-where
+) where
 
 import Control.Applicative
-
+import Prelude
 
 #if defined(mingw32_HOST_OS) || defined(__MINGW32__)
-
 import Data.Word
-
 #else
-
 import System.Posix.Process
-
 #endif
 
+getPID :: IO Int
 
 #if defined(mingw32_HOST_OS) || defined(__MINGW32__)
--- This function is not yet in the win32 package, so we have to 
+-- This function is not yet in the win32 package, so we have to
 -- roll down our own definition.
 --
 -- Credit goes where it is deserved:
@@ -31,12 +25,7 @@ import System.Posix.Process
 foreign import stdcall unsafe "winbase.h GetCurrentProcessId"
     c_GetCurrentProcessId :: IO Word32
 
-getPID :: IO Int
 getPID = fromIntegral <$> c_GetCurrentProcessId
-
 #else
-
-getPID :: IO Int
 getPID = fromIntegral <$> getProcessID
-
 #endif
