@@ -18,8 +18,8 @@ import Data.List
 
 import Control.Arrow ((***))
 
-import Control.Monad       (liftM, filterM, unless, guard, foldM, (>=>))
-import Control.Monad.Trans (liftIO)
+import Control.Monad (filterM, unless, guard, foldM)
+import Control.Monad.IO.Class (liftIO)
 import Control.Monad.Catch
 
 import Hint.Base
@@ -32,11 +32,9 @@ import System.Random
 import System.FilePath
 import System.Directory
 
-#if defined(NEED_PHANTOM_DIRECTORY)
 import Data.Maybe (maybe)
 import Hint.Configuration (setGhcOption)
 import System.IO.Temp
-#endif
 
 type ModuleText = String
 
@@ -62,7 +60,6 @@ newPhantomModule =
 
 getPhantomDirectory :: MonadInterpreter m => m FilePath
 getPhantomDirectory =
-#if defined(NEED_PHANTOM_DIRECTORY)
     -- When a module is loaded by file name, ghc-8.4.1 loses track of the
     -- file location after the first time it has been loaded, so we create
     -- a directory for the phantom modules and add it to the search path.
@@ -74,15 +71,14 @@ getPhantomDirectory =
                          onState (\s -> s{ phantomDirectory = Just fp })
                          setGhcOption $ "-i" ++ fp
                          return fp
-#else
-    liftIO getTemporaryDirectory
-#endif
 
 allModulesInContext :: MonadInterpreter m => m ([ModuleName], [ModuleName])
 allModulesInContext = runGhc getContextNames
 
 getContext :: GHC.GhcMonad m => m ([GHC.Module], [GHC.ImportDecl GHC.GhcPs])
-getContext = GHC.getContext >>= foldM f ([], [])
+getContext = do
+    ctx <- GHC.getContext
+    foldM f ([], []) ctx
   where
     f :: (GHC.GhcMonad m) =>
          ([GHC.Module], [GHC.ImportDecl GHC.GhcPs]) ->
@@ -90,15 +86,11 @@ getContext = GHC.getContext >>= foldM f ([], [])
          m ([GHC.Module], [GHC.ImportDecl GHC.GhcPs])
     f (ns, ds) i = case i of
       (GHC.IIDecl d)     -> return (ns, d : ds)
-      m@(GHC.IIModule _) -> do n <- iiModToMod m; return (n : ns, ds)
+      (GHC.IIModule m) -> do n <- GHC.findModule m Nothing; return (n : ns, ds)
 
 modToIIMod :: GHC.Module -> GHC.InteractiveImport
 modToIIMod = GHC.IIModule . GHC.moduleName
 
-iiModToMod :: GHC.GhcMonad m => GHC.InteractiveImport -> m GHC.Module
-iiModToMod (GHC.IIModule m) = GHC.findModule m Nothing
-iiModToMod _ = error "iiModToMod!"
-
 getContextNames :: GHC.GhcMonad m => m([String], [String])
 getContextNames = fmap (map name *** map decl) getContext
     where name = GHC.moduleNameString . GHC.moduleName
@@ -149,7 +141,7 @@ addPhantomModule mod_text =
        --
        return pm
 
-removePhantomModule :: MonadInterpreter m => PhantomModule -> m ()
+removePhantomModule :: forall m. MonadInterpreter m => PhantomModule -> m ()
 removePhantomModule pm =
     do -- We don't want to actually unload this module, because that
        -- would mean that all the real modules might get reloaded and the
@@ -168,9 +160,10 @@ removePhantomModule pm =
                      let mods' = filter (mod /=) mods
                      runGhc2 setContext mods' imps
                      --
-                     let isNotPhantom = isPhantomModule . moduleToString  >=>
-                                          return . not
-                     null `liftM` filterM isNotPhantom mods'
+                     let isNotPhantom :: GHC.Module -> m Bool
+                         isNotPhantom mod' = do
+                           not <$> isPhantomModule (moduleToString mod')
+                     null <$> filterM isNotPhantom mods'
              else return True
        --
        let file_name = pmFile pm
@@ -235,22 +228,24 @@ doLoad fs = mayFail $ do
 
 -- | Returns True if the module was interpreted.
 isModuleInterpreted :: MonadInterpreter m => ModuleName -> m Bool
-isModuleInterpreted m = findModule m >>= runGhc1 GHC.moduleIsInterpreted
+isModuleInterpreted moduleName = do
+  mod <- findModule moduleName
+  runGhc1 GHC.moduleIsInterpreted mod
 
 -- | Returns the list of modules loaded with 'loadModules'.
 getLoadedModules :: MonadInterpreter m => m [ModuleName]
 getLoadedModules = do (active_pms, zombie_pms) <- getPhantomModules
-                      ms <- map modNameFromSummary `liftM` getLoadedModSummaries
+                      ms <- map modNameFromSummary <$> getLoadedModSummaries
                       return $ ms \\ map pmName (active_pms ++ zombie_pms)
 
 modNameFromSummary :: GHC.ModSummary -> ModuleName
 modNameFromSummary = moduleToString . GHC.ms_mod
 
 getLoadedModSummaries :: MonadInterpreter m => m [GHC.ModSummary]
-getLoadedModSummaries =
-  do all_mod_summ <- runGhc GHC.getModuleGraph
-     filterM (runGhc1 GHC.isLoaded . GHC.ms_mod_name)
-             (GHC.mgModSummaries all_mod_summ)
+getLoadedModSummaries = do
+    modGraph <- runGhc GHC.getModuleGraph
+    let modSummaries = GHC.mgModSummaries modGraph
+    filterM (runGhc1 GHC.isLoaded . GHC.ms_mod_name) modSummaries
 
 -- | Sets the modules whose context is used during evaluation. All bindings
 --   of these modules are in scope, not only those exported.
@@ -302,32 +297,41 @@ setImportsQ ms = setImportsF $ map (\(m,q) -> ModuleImport m (maybe NotQualified
 --   @setImportsF [ModuleImport "Prelude" NotQualified NoImportList, ModuleImport "Data.Text" (QualifiedAs $ Just "Text") (HidingList ["pack"])]@
 
 setImportsF :: MonadInterpreter m => [ModuleImport] -> m ()
-setImportsF ms = do
+setImportsF moduleImports = do
        regularMods <- mapM (findModule . modName) regularImports
        mapM_ (findModule . modName) phantomImports -- just to be sure they exist
        --
        old_qual_hack_mod <- fromState importQualHackMod
        maybe (return ()) removePhantomModule old_qual_hack_mod
        --
-       new_pm <- if null phantomImports
-                   then return Nothing
-                   else do
-                     new_pm <- addPhantomModule $ \mod_name -> unlines $
-                                ("module " ++ mod_name ++ " where ") :
-                                map newImportLine phantomImports
-                     onState (\s -> s{importQualHackMod = Just new_pm})
-                     return $ Just new_pm
+       maybe_phantom_module <- do
+         if null phantomImports
+           then return Nothing
+           else do
+             let moduleContents = map newImportLine phantomImports
+             new_phantom_module <- addPhantomModule $ \mod_name
+               -> unlines $ ("module " ++ mod_name ++ " where ")
+                          : moduleContents
+             onState (\s -> s{importQualHackMod = Just new_phantom_module})
+             return $ Just new_phantom_module
        --
-       pm <- maybe (return []) (findModule . pmName >=> return . return) new_pm
+       phantom_mods <- case maybe_phantom_module of
+         Nothing -> do
+           pure []
+         Just phantom_module-> do
+           phantom_mod <- findModule (pmName phantom_module)
+           pure [phantom_mod]
        (old_top_level, _) <- runGhc getContext
-       let new_top_level = pm ++ old_top_level
+       let new_top_level = phantom_mods ++ old_top_level
        runGhc2 setContextModules new_top_level regularMods
        --
        onState (\s ->s{qualImports = phantomImports})
   where
-    (regularImports, phantomImports) = partitionEithers $ map (\m -> if isQualified m || hasImportList m
-                                                                       then Right m
-                                                                       else Left m) ms
+    (regularImports, phantomImports) = partitionEithers
+                                     $ map (\m -> if isQualified m || hasImportList m
+                                                  then Right m  -- phantom
+                                                  else Left m)
+                                           moduleImports
     isQualified m = modQual m /= NotQualified
     hasImportList m = modImp m /= NoImportList
     newImportLine m = concat ["import ", case modQual m of
@@ -367,11 +371,10 @@ cleanPhantomModules =
                         importQualHackMod = Nothing,
                         qualImports         = []})
        liftIO $ mapM_ (removeFile . pmFile) (old_active ++ old_zombie)
-#if defined(NEED_PHANTOM_DIRECTORY)
+
        old_phantomdir <- fromState phantomDirectory
        onState (\s -> s{phantomDirectory    = Nothing})
        liftIO $ do maybe (return ()) removeDirectory old_phantomdir
-#endif
 
 -- | All imported modules are cleared from the context, and
 --   loaded modules are unloaded. It is similar to a @:load@ in
