@@ -12,7 +12,10 @@ import Hint.Context
 import Hint.Configuration
 import Hint.Extension
 
-import Control.Monad.Reader
+import Control.Monad (ap, unless)
+import Control.Monad.IO.Class
+import Control.Monad.Trans.Class
+import Control.Monad.Trans.Reader
 import Control.Monad.Catch as MC
 
 import Data.Typeable (Typeable)
@@ -88,7 +91,9 @@ initialize args =
        -- available; calling this function once is mandatory!
        _ <- runGhc1 GHC.setSessionDynFlags df2{GHC.log_action = log_handler}
 
-       let extMap      = map (\fs -> (GHC.flagSpecName fs, GHC.flagSpecFlag fs)) GHC.xFlags
+       let extMap      = [ (GHC.flagSpecName flagSpec, GHC.flagSpecFlag flagSpec)
+                         | flagSpec <- GHC.xFlags
+                         ]
        let toOpt e     = let err = error ("init error: unknown ext:" ++ show e)
                          in fromMaybe err (lookup e extMap)
        let getOptVal e = (asExtension e, GHC.xopt (toOpt e) df2)
@@ -123,23 +128,25 @@ runInterpreterWithArgsLibdir :: (MonadIO m, MonadMask m)
                              -> InterpreterT m a
                              -> m (Either InterpreterError a)
 runInterpreterWithArgsLibdir args libdir action =
+#ifndef THREAD_SAFE_LINKER
   ifInterpreterNotRunning $
+#endif
     do s <- newInterpreterSession `MC.catch` rethrowGhcException
        execute libdir s (initialize args >> action `finally` cleanSession)
     where rethrowGhcException   = throwM . GhcException . showGhcEx
           newInterpreterSession = newSessionData ()
           cleanSession = cleanPhantomModules
 
+#ifndef THREAD_SAFE_LINKER
 {-# NOINLINE uniqueToken #-}
 uniqueToken :: MVar ()
 uniqueToken = unsafePerformIO $ newMVar ()
 
 ifInterpreterNotRunning :: (MonadIO m, MonadMask m) => m a -> m a
-ifInterpreterNotRunning action =
-    do maybe_token <- liftIO $ tryTakeMVar uniqueToken
-       case maybe_token of
-           Nothing -> throwM MultipleInstancesNotAllowed
-           Just x  -> action `finally` liftIO (putMVar uniqueToken x)
+ifInterpreterNotRunning action = liftIO (tryTakeMVar uniqueToken) >>= \ case
+    Nothing -> throwM MultipleInstancesNotAllowed
+    Just x  -> action `finally` liftIO (putMVar uniqueToken x)
+#endif
 
 -- | The installed version of ghc is not thread-safe. This exception
 --   is thrown whenever you try to execute @runInterpreter@ while another
@@ -156,9 +163,7 @@ initialState :: InterpreterState
 initialState = St {
                    activePhantoms    = [],
                    zombiePhantoms    = [],
-#if defined(NEED_PHANTOM_DIRECTORY)
                    phantomDirectory  = Nothing,
-#endif
                    hintSupportModule = error "No support module loaded!",
                    importQualHackMod = Nothing,
                    qualImports       = [],
