@@ -4,7 +4,7 @@ import Prelude hiding (span)
 
 import Hint.Base
 
-import Control.Monad.Trans (liftIO)
+import Control.Monad.IO.Class (liftIO)
 
 import qualified Hint.GHC as GHC
 
@@ -29,25 +29,25 @@ runParser parser expr =
        case parse_res of
            GHC.POk{}            -> return ParseOk
            --
-#if __GLASGOW_HASKELL__ >= 804
-           GHC.PFailed _ span err
+#if __GLASGOW_HASKELL__ >= 810
+           GHC.PFailed pst      -> let errMsgs = GHC.getErrorMessages pst dyn_fl
+                                       span = foldr (GHC.combineSrcSpans . GHC.errMsgSpan) GHC.noSrcSpan errMsgs
+                                       err = GHC.vcat $ GHC.pprErrMsgBagWithLoc errMsgs
+                                   in pure (ParseError span err)
 #else
-           GHC.PFailed span err
-#endif
+           GHC.PFailed _ span err
                                 -> return (ParseError span err)
+#endif
 
 failOnParseError :: MonadInterpreter m
                  => (String -> m ParseResult)
                  -> String
                  -> m ()
 failOnParseError parser expr = mayFail go
-    where go = do parsed <- parser expr
-                  --
-                  -- If there was a parsing error,
-                  -- do the "standard" error reporting
-                  case parsed of
+    where go = parser expr >>= \ case
                       ParseOk             -> return (Just ())
-                      --
+                      -- If there was a parsing error,
+                      -- do the "standard" error reporting
                       ParseError span err ->
                           do -- parsing failed, so we report it just as all
                              -- other errors get reported....
