@@ -2,13 +2,14 @@ module Main (main) where
 
 import Prelude hiding (catch)
 
-import Control.Exception.Extensible (ArithException(..), AsyncException(UserInterrupt))
+import Control.Exception (ArithException(..), AsyncException(UserInterrupt))
 import Control.Monad.Catch as MC
 
-import Control.Monad (liftM, when, void, (>=>))
+import Control.Monad (guard, liftM, when, void, (>=>))
 
 import Control.Concurrent (forkIO, threadDelay)
 import Control.Concurrent.MVar
+import Control.Concurrent.STM
 
 import Data.IORef
 
@@ -21,7 +22,7 @@ import System.Exit
 import System.Posix.Signals
 #endif
 
-import Test.HUnit ((@?=), (@?))
+import Test.HUnit ((@?=), (@?), assertFailure)
 import qualified Test.HUnit as HUnit
 
 import Language.Haskell.Interpreter
@@ -204,6 +205,13 @@ test_catch = TestCase "catch" [] $ do
           action = do s <- eval "1 `div` 0 :: Int"
                       return $! s
 
+#ifndef THREAD_SAFE_LINKER
+-- Prior to ghc-8.10, the linker wasn't thread-safe, and so running multiple
+-- instances of hint in different threads can lead to mysterious errors of the
+-- form "Could not load '*_closure', dependency unresolved". To make that error
+-- less mysterious, 'ifInterpreterNotRunning' throw a clearer error earlier, as
+-- soon as it detects that the user is trying to run multiple instances of hint
+-- in parallel. This test ensures that this nicer error is thrown.
 test_only_one_instance :: TestCase
 test_only_one_instance = TestCase "only_one_instance" [] $ liftIO $ do
         r <- newEmptyMVar
@@ -213,6 +221,59 @@ test_only_one_instance = TestCase "only_one_instance" [] $ liftIO $ do
                                        return $ Right ()
         _ <- forkIO $ Control.Monad.void concurrent
         readMVar r @?  "concurrent instance did not fail"
+#else
+-- Prior to ghc-8.10, the linker wasn't thread-safe, and so running multiple
+-- instances of hint in different threads can lead to mysterious errors of the
+-- form "Could not load '*_closure', dependency unresolved". This test ensures
+-- that this error no longer occurs. The important thing about this test is
+-- that it should _fail_ if 'ifInterpreterNotRunning' is deleted and this test
+-- is run on an older ghc version. Otherwise this test is not testing what it's
+-- meant to.
+test_multiple_instances :: TestCase
+test_multiple_instances = TestCase "multiple_instances" ["mod_file"] $ liftIO $ do
+        writeFile mod_file "f = id"
+
+        -- ensure the two threads interleave in a deterministic way
+        tvar <- newTVarIO 1
+        let step n = liftIO $ atomically $ do
+              nextStep <- readTVar tvar
+              guard (nextStep >= n)
+              writeTVar tvar (n + 1)
+            skipToStep n = liftIO $ atomically $ do
+              modifyTVar tvar (max n)
+
+        mvar1 <- newEmptyMVar
+        mvar2 <- newEmptyMVar
+        void $ forkIO $ do
+          r1 <- try $ runInterpreter $ do
+            step 1
+            loadModules [mod_file]
+            step 3
+            setTopLevelModules ["Main"]
+            step 5
+            setImports ["Prelude"]
+            step 7
+            eval "f [1,2]" @@?= "[1,2]"
+            step 9
+          skipToStep 9
+          putMVar mvar1 r1
+        void $ forkIO $ do
+          r2 <- try $ runInterpreter $ do
+            step 2
+            loadModules [mod_file]
+            step 4
+            setTopLevelModules ["Main"]
+            step 6
+            setImports ["Prelude"]
+            step 8
+            eval "f [1,2]" @@?= "[1,2]"
+            step 10
+          skipToStep 10
+          putMVar mvar2 r2
+        noInterpreterError =<< noExceptions =<< takeMVar mvar1
+        noInterpreterError =<< noExceptions =<< takeMVar mvar2
+    where mod_file = "TEST_MultipleInstances.hs"
+#endif
 
 test_normalize_type :: TestCase
 test_normalize_type = TestCase "normalize_type" [mod_file] $ do
@@ -283,7 +344,11 @@ tests = [test_reload_modified
         ,test_search_path
         ,test_search_path_dot
         ,test_catch
+#ifndef THREAD_SAFE_LINKER
         ,test_only_one_instance
+#else
+        ,test_multiple_instances
+#endif
         ,test_normalize_type
         ]
 
@@ -330,6 +395,14 @@ fails action = (action >> return False) `catchIE` (\_ -> return True)
 succeeds :: (MonadCatch m, MonadIO m) => m a -> m Bool
 succeeds = fmap not . fails
 
+noExceptions :: Either SomeException a -> IO a
+noExceptions (Left  e) = assertFailure (show e)
+noExceptions (Right a) = pure a
+
+noInterpreterError :: Either InterpreterError a -> IO a
+noInterpreterError (Left  e) = assertFailure (show e)
+noInterpreterError (Right a) = pure a
+
 data IOTestCase = IOTestCase String [FilePath] ((Interpreter () -> IO (Either InterpreterError ())) -> IO (Either InterpreterError ()))
 
 runIOTests :: Bool -> [IOTestCase] -> IO HUnit.Counts
@@ -340,8 +413,7 @@ runIOTests sandboxed = HUnit.runTestTT . HUnit.TestList . map build
                   clean_up = mapM_ removeIfExists tmps
                   go       = do r <- test (\body -> runInterpreter
                                             (when sandboxed setSandbox >> body))
-                                either (printInterpreterError >=> (fail . show))
-                                       return r
+                                noInterpreterError r
                   removeIfExists f = do existsF <- doesFileExist f
                                         if existsF
                                           then removeFile f
